import processing.serial.*;
Serial myArduino;
float pulse;

float e = 100, targetE = e;


/*********** 카메라 관련 변수 ***********/
import gohai.glvideo.*;
GLCapture cam;
int camW = 320, camH = 240;

/*********** 작동 관련 ***********/
boolean activate = false;
boolean blink = false;
int timeNow = 0;
int timePast = 0;

int rectW = 50;
int max = 600;
float a = max, targetA = a;
float c = max, targetC = c;
float d = max, targetD = d;
float f = max, targetF = f;
float h = max, targetH = h;
float n = max, targetN = n;
float s = max, targetS = s;
float p = max, targetP = p;

/*********** 이미지 관련 변수 ***********/
PImage sample;
String fileName;  //이피지 파일의 이름
String fileDir;   //이미지 파일의 경로
String file;      //이미지의 경로 + 이름

/*********** 서버의 응답 관련 변수 ***********/
String postResult;  //서버의 응답을 저장할 변수

/*********** 이미지 시각화 관련 변수 ***********/
PFont font;

//JSONObject jsonObject;

void setup() {

  printArray(Serial.list());
  String myPort = Serial.list() [1];
  myArduino = new Serial(this, myPort, 9600);


  size(1000, 700, P2D);
  println("AZURE Face::Detect");

  //서버에 전송할 파일을 특정하여 불러옴
  fileName = "sample.jpg";
  fileDir = sketchPath()+"/data/"; 
  file = fileDir+fileName; 
  sample = loadImage(file);

  //화면에 표시할 글꼴 설정
  font = createFont("Noto Mono", 15);
  textFont(font);

  //카메라 설정 
  String[] cameras = GLCapture.list();
  println("cameras:");
  printArray(cameras);
  if (0 < cameras.length) {
    String[] configs = GLCapture.configs(cameras[0]);
    println("Configs:");
    printArray(configs);
  }
  cam = new GLCapture(this);
  cam.start();
}
void draw() {
  background(0);
  e += (targetE-e)*0.1;
  fill(255,0,0,90);
  ellipse(650,height/2, e, e);
  
  if (myArduino.available()>0) {
    String serialIn = myArduino.readStringUntil('\n');
    if (serialIn != null){
      serialIn = trim(serialIn);
      println(serialIn);
      int data = int(serialIn);
      targetE = map(data, 0, 1023, 10, 500);
    }
  }
  timeNow = second();

  a += (targetA-a)*0.1;
  c += (targetC-c)*0.1;
  d += (targetD-d)*0.1;
  f += (targetF-f)*0.1;
  h += (targetH-h)*0.1;
  n += (targetN-n)*0.1;
  s += (targetS-s)*0.1;
  p += (targetP-p)*0.1;


  //카메라 이미지를 화면에 표시함
  if (cam.available() == true) {
    cam.read();
  }
  image(cam, 0, 0, camW, camH);

  if (timeNow != timePast) {
    println("timeNow", timeNow);
  }

  if (timeNow%5 == 0 && timeNow != timePast) {
    println("frameCount", frameCount);
    saveCamImage(cam);           //카메라 이미지를 저장
    thread("postImage");         //서버의 주소(url)로 이미지를 전송하고 서버의 응답을 회수하는 작업을 Thread로 처리
  }


  timePast = timeNow;

  /* if (activate) {                //작동 시작하면
   saveCamImage(cam);           //카메라 이미지를 저장
   thread("postImage");         //서버의 주소(url)로 이미지를 전송하고 서버의 응답을 회수하는 작업을 Thread로 처리함
   activate = false;            //작동 정지
   }*/

  //서버가 응답을 했을 경우, 응답을 분석하여 화면에 시각화함
  if (postResult != null && postResult != "") {
    JSONArray JSONvalues = parseJSONArray(postResult);
    displayFaceDetectResults(JSONvalues);
    //float anger = jsonObject.getJSONObject("faceAttributes").getJSONObject("emotion").getFloat("anger");
    blink = false;
    fill(255, 0, 0);
    //rect(30,10,10,anger);
  } 
  if (blink) {
    if (frameCount % 10 < 5) {
      fill(255, 0, 0);
    } else {
      fill(255);
    }
    noStroke();
    ellipse(10, 10, 10, 10);
  }
}

void mousePressed() {          //마우스를 누르면,
  activate = true;             //작동 시작
  blink = true;                //깜빡임 시작
  postResult = "";             //이전의 인식 결과 제거
}

void saveCamImage(PImage camImage) {
  PImage sample = camImage.get(); 
  sample.save(file);
}

void postImage() {
  // Azure가 정의한 규약에 따라 POST Request 구조화
  PostRequest post = new PostRequest(url);
  post.addHeader("Content-Type", "application/octet-stream");  //Request Header
  post.addHeader("Ocp-Apim-Subscription-Key", apiKey);         //Request Header
  post.addDataFromFile(file);                                  //Request Body
  post.send();                                                 //서버에 POST 방식으로 전송

  postResult = post.getContent();                       //서버의 응답 회수
  println(postResult);                                         //회수한 응답 출력
  //return postResult;                                           //응답 반환
}

//JSON 객체를 분석하여 데이터를 추출하고 시각화
void displayFaceRectangle(JSONObject jsonObject) {
  int y = jsonObject.getJSONObject("faceRectangle").getInt("top");
  int x = jsonObject.getJSONObject("faceRectangle").getInt("left");
  int w = jsonObject.getJSONObject("faceRectangle").getInt("width");
  int h = jsonObject.getJSONObject("faceRectangle").getInt("height");
  String gender = jsonObject.getJSONObject("faceAttributes").getString("gender");
  int age = jsonObject.getJSONObject("faceAttributes").getInt("age");

  float anger = jsonObject.getJSONObject("faceAttributes").getJSONObject("emotion").getFloat("anger");
  float contempt = jsonObject.getJSONObject("faceAttributes").getJSONObject("emotion").getFloat("contempt");
  float disgust = jsonObject.getJSONObject("faceAttributes").getJSONObject("emotion").getFloat("disgust");
  float fear = jsonObject.getJSONObject("faceAttributes").getJSONObject("emotion").getFloat("fear");
  float happiness = jsonObject.getJSONObject("faceAttributes").getJSONObject("emotion").getFloat("happiness");
  float neutral = jsonObject.getJSONObject("faceAttributes").getJSONObject("emotion").getFloat("neutral");
  float sadness = jsonObject.getJSONObject("faceAttributes").getJSONObject("emotion").getFloat("sadness");
  float surprise = jsonObject.getJSONObject("faceAttributes").getJSONObject("emotion").getFloat("surprise");


  println(contempt, disgust, fear, happiness, neutral, sadness, surprise);

  stroke(0, 255, 0);
  strokeWeight(3);
  noFill();
  rect(x, y, w, h);
  fill(0, 255, 0);
  textSize(20);
  //text(gender+"("+age+")", x, y-h/10);
  text("happy"+happiness, x, y-h/10);

  targetA = map(anger, 0, 1, 0, max);
  targetC = map(contempt, 0, 1, 0, max);
  targetD = map(disgust, 0, 1, 0, max);
  targetF = map(fear, 0, 1, 0, max);
  targetH = map(happiness, 0, 1, 0, max);
  targetN = map(neutral, 0, 1, 0, max);
  targetS = map(sadness, 0, 1, 0, max);
  targetP = map(surprise, 0, 1, 0, max);

  noStroke();
  textAlign(CENTER);
  textSize(15);

  fill(206,33,33);
  rect(340, 30, rectW, a);
  text("anger", 365, 25);

  fill(111,88,70);
  rect(420, 30, rectW, c);
  text("contempt", 445, 25);

  fill(118,94,129);
  rect(500, 30, rectW, d);
  text("disgust", 525, 25);

  fill(154,222,234);
  rect(580, 30, rectW, f);
  text("fear", 605, 25);

  fill(252,153,166);
  rect(660, 30, rectW, h);
  text("happiness", 685, 25);

  fill(255);
  rect(740, 30, rectW, n);
  text("neutral", 765, 25);

  fill(32,49,188);
  rect(820, 30, rectW, s);
  text("sadness", 845, 25);

  fill(255,236,139);
  rect(900, 30, rectW, p);
  text("surprise", 925, 25);
}

//서버로 부터 회수한 JSON 배열의 각 객체를 displayFaceRectangle() 함수로 처리
void displayFaceDetectResults(JSONArray jsonArray) {
  for (int i=0; i<jsonArray.size(); i++) {
    JSONObject jsonObject = jsonArray.getJSONObject(i);
    displayFaceRectangle(jsonObject);
  }
}
